/*
 * ===========================
 * VDK Visual Development Kit
 * Version 0.4
 * October 1998
 * ===========================
 *
 * Copyright (C) 1998, Mario Motta
 * Developed by Mario Motta <mmotta@guest.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-130
 */ 

#ifndef VDKSTRING_H
#define VDKSTRING_H
#include <string.h>

struct STRING {          
  char* s;              
  unsigned int ref ;    
};

/*!
  \class VDKString
  \brief Implements famous cont referenced string objects
 */
class VDKString {       
 protected:
  STRING* p;         
 public:

  /*!
    Constructor, makes an empty string
    \code
    VDKString s;
    \endcode
   */
  VDKString() 
    {
      p = new STRING; p->s = new char;
      *(p->s) = (char) 0; p->ref = 1;
    }
  /*!
    Constructor
    \param s a null terminated C string
    \code
    VDKString s = "uncle bill";
    \endcode
   */

  VDKString (char*s) 
    {
      p = new STRING;
      if(s)
	{
	  p->s = new char[strlen(s)+1];
	  strcpy(p->s,s);
	}
      else
	{
	p->s = new char;
	*(p->s) = (char) 0;
	}
      p->ref = 1; 
    }
  /*!
    Copy-initializer
    \param s a VDKString reference
    \code
    VDKString s = "uncle bill";
    VDKString s1 = s;
    \endcode
   */

  VDKString(const VDKString& s)
    {
      s.p-> ref++;         
      p = s.p;             
    }

  /*!
    Assignement operator
    \param s a VDKString reference
    \code
    VDKString s = "uncle bill";
    VDKString s1 = s;
    VDKString s2 = "uncle sam";
    s = s2;
    \endcode
   */
  VDKString& operator= (const VDKString& s) 
    {
      if (this == &s) return *this;
      else s.p->ref++;
      if (--p->ref == 0) 
	{ 
	  delete p->s;delete p; 
	} 
      p= s.p; 
      return *this;
    }

  /*!
    Assignement operator
    \param s a null terminated C string
    \code
    VDKString s = "uncle bill";
    s = "uncle sam";
    \endcode
   */
  VDKString& operator= (char* s){
    if (p->ref > 1) {
      p->ref--;
      p = new STRING;}
    else if (p->ref == 1) delete p->s;
    if(s)
	{
	  p->s = new char[strlen(s)+1];
	  strcpy(p->s,s);
	}
      else
	{
	p->s = new char;
	*(p->s) = (char) 0;
	}
    p->ref = 1;
    return *this;
  }

  /*!
    Destructor
   */
  ~VDKString() {
    if(--p->ref == 0)
      {
	delete[] p->s;
	delete p;  
      }
  }

  /*!
    VDKString to char* casting
    ** warning ** can violate data hiding OO concept
    */
  operator char*() {return p->s; }

  /*!
    Equality operator
  */
  int operator == (VDKString& s) { return !strcmp(p->s,s.p->s); }
  /*!
    less than operator
  */
  int operator <(VDKString& s) {return strcmp(p->s,s.p->s) < 0; }
  /*!
    greater than operator
  */
  int operator>(VDKString& s) { return strcmp(p->s,s.p->s) > 0; }
  /*!
    less-equal operator
  */
  int operator <=(VDKString& s) { return (*this < s || *this == s); }
  /*!
    greater-equal operator
  */
  int operator >=(VDKString& s) { return (*this > s || *this == s); }
  /*!
    disequality operator
  */
  int operator !=(VDKString& s) { return !(*this == s);	 }
  /*!
    cat to this
    \param s a null terminated string
    \code
    VDKString s = "uncle bill";
    s += " is a smart boy";
    \endcode
  */
  VDKString& operator +=(char* s) 
  {
    char* local = new char[strlen(p->s)+strlen (s)+1];
    strcat(strcpy(local,p->s),s);  
    *this = local;
    return *this;
  }
  /*!
    cat to this
    \param s a VDKString
    \code
    VDKString s = "uncle bill";
    VDKString s1 = " is a smart boy";
    s += s1;
    \endcode
  */
  VDKString& operator +=(VDKString& s)
  {
    // riutilizzo +=
    *this += (char*) s;
    return *this;		
  }
  /*!
    Returns a VDKString concatenated
    \param s a null terminated string
    \code
    VDKString s = "uncle bill";
    VDKString s1 = s + " is a smart boy";
    \endcode
   */
  VDKString  operator + (char* s);
  /*!
    Returns a VDKString concatenated
    \param s a VDKString
   */
  VDKString  operator +(VDKString& s){ return *this + (char*) s;	}
  /*!
    Returns true if this is an empty string
   */
  bool isNull() { return !(*p->s); }
  /*!
    as strlen()
  */
  int size() { return strlen(p->s); }
};
#endif




